@@ -1,3 +1,5 @@
+import warnings
+
 from collections import defaultdict
 from copy import copy
 
@@ -8,8 +10,10 @@ from django.db.models import signals
 from django.utils.http import urlquote
 from django.utils.translation import get_language
 from django.template.defaulttags import url as url_tag
+from django.template import Context
 
 from models import Tree, TreeItem
+from utils import DJANGO_VERSION_INT
 
 # Sitetree objects are stored in Django cache for a year (60 * 60 * 24 * 365 = 31536000 sec).
 # Cache is only invalidated on sitetree or sitetree item change.
@@ -87,12 +91,35 @@ def register_i18n_trees(aliases):
     _I18N_TREES = aliases
 
 
+class LazyTitle(object):
+    """Lazily resolves any variable found in a title of an item.
+    Produces resolved title as unicode representation."""
+
+    def __init__(self, title):
+        self.title = title
+
+    def __unicode__(self):
+        my_lexer = template.Lexer(self.title, template.UNKNOWN_SOURCE)
+        my_tokens = my_lexer.tokenize()
+
+        # Deliberately strip off template tokens that are not text or variable.
+        for my_token in my_tokens:
+            if my_token.token_type not in (template.TOKEN_TEXT, template.TOKEN_VAR):
+                my_tokens.remove(my_token)
+
+        my_parser = template.Parser(my_tokens)
+        return my_parser.parse().render(SiteTree.get_global_context())
+
+    def __eq__(self, other):
+        return self.__unicode__() == other
+
+
 class SiteTree(object):
 
+    _global_context = Context()
+
     def __init__(self):
         self.cache = None
-        # This points to global sitetree context.
-        self.global_context = None
         # Listen for signals from the models.
         signals.post_save.connect(self.cache_empty, sender=Tree)
         signals.post_save.connect(self.cache_empty, sender=TreeItem)
@@ -132,13 +159,19 @@ class SiteTree(object):
         """Replaces entire cache entry parameter data by its name with new data."""
         self.cache[entry_name][key] = value
 
-    def set_global_context(self, context):
+    @classmethod
+    def set_global_context(cls, context):
         """Saves context as global context if not already set or if changed.
         Almost all variables are resolved against global context.
 
         """
-        if not self.global_context or hash(context) != hash(self.global_context):
-            self.global_context = context
+        if not cls._global_context or hash(context) != hash(cls._global_context):
+            cls._global_context = context
+
+    @classmethod
+    def get_global_context(cls):
+        """Returns current sitetree global context."""
+        return cls._global_context
 
     def resolve_tree_i18n_alias(self, alias):
         """Resolves internationalized tree alias.
@@ -164,7 +197,7 @@ class SiteTree(object):
         """
         self.cache_init()
         sitetree_needs_caching = False
-        if self.global_context.current_app != 'admin':
+        if self._global_context.current_app != 'admin':
             # We do not need i18n for a tree rendered in Admin dropdown.
             alias = self.resolve_tree_i18n_alias(alias)
         sitetree = self.get_cache_entry('sitetrees', alias)
@@ -202,14 +235,15 @@ class SiteTree(object):
                                                item.access_permissions.select_related()])
             # Contextual properties.
             item.url_resolved = self.url(item)
-            item.title_resolved = item.title
+            if template.VARIABLE_TAG_START in item.title:
+                item.title_resolved = LazyTitle(item.title)
+            else:
+                item.title_resolved = item.title
             item.is_current = False
             item.in_current_branch = False
 
         # Get current item for the given sitetree.
         self.get_tree_current_item(alias)
-        # Parse titles.
-        self.parse_titles(sitetree)
 
         # Save sitetree data into cache if needed.
         if sitetree_needs_caching:
@@ -238,11 +272,11 @@ class SiteTree(object):
         """
         current_item = None
 
-        if 'request' not in self.global_context and self.global_context.current_app != 'admin':
+        if 'request' not in self._global_context and self._global_context.current_app != 'admin':
             raise SiteTreeError('Sitetree needs "django.core.context_processors.request" to be in TEMPLATE_CONTEXT_PROCESSORS in your settings file. If it is, check that your view pushes request data into the template.')
         else:
             # urlquote is a try to support non-ascii in url.
-            current_url = urlquote(self.global_context['request'].path)
+            current_url = urlquote(self._global_context['request'].path)
             urls_cache = self.get_cache_entry('urls', tree_alias)
             if urls_cache:
                 for url_item in urls_cache:
@@ -255,7 +289,7 @@ class SiteTree(object):
 
         return current_item
 
-    def url(self, sitetree_item, tag_arguments=[], context=None):
+    def url(self, sitetree_item, tag_arguments=None, context=None):
         """Resolves item's URL.
 
         'sitetree_item' points to TreeItem object, 'url' property of which
@@ -265,8 +299,14 @@ class SiteTree(object):
             'sitetree_url' in template.
 
         """
+        if tag_arguments is None:
+            tag_arguments = []
+        else:
+            # TODO Remove tag_arguments in 1.0.
+            warnings.warn('Use of sitetree_url tag additional arguments is deprecated. Feature support will be completely removed in 1.0.', DeprecationWarning)
+
         if context is None:
-            context = self.global_context
+            context = self._global_context
 
         if not isinstance(sitetree_item, TreeItem):
             sitetree_item = self.resolve_var(sitetree_item, context)
@@ -274,46 +314,50 @@ class SiteTree(object):
         # Resolve only if item's URL is marked as pattern.
         if sitetree_item.urlaspattern:
             resolved_var = self.resolve_var(sitetree_item.url, context)
-            if isinstance(resolved_var, list):
-                raise SiteTreeError('Named URL for "%s" sitetree item clashes with template variable name. Please change either name.' % sitetree_item.title)
-            view_path = resolved_var.split(' ')
-            # We should try to resolve URL parameters from site tree item.
+
+            # Check whether a template variable is used instead of a URL pattern.
+            if resolved_var != sitetree_item.url:
+                if not isinstance(resolved_var, basestring):  # Variable contains what we're not expecting, revert to original URL.
+                    resolved_var = sitetree_item.url
+                # TODO Remove template var resolution in 1.0.
+                warnings.warn('Use of a template variable in URL field is deprecated. Feature support will be completely removed in 1.0.', DeprecationWarning)
+
+            view_path = resolved_var
+            all_arguments = copy(tag_arguments)
+
+            if ' ' in resolved_var:
+                view_path = resolved_var.split(' ')
+                # We should try to resolve URL parameters from site tree item.
+                for view_argument in view_path[1:]:
+                    resolved = self.resolve_var(view_argument)
+                    # In case of non-ascii data we leave variable unresolved.
+                    if isinstance(resolved, unicode):
+                        if resolved.encode('ascii', 'ignore').decode('ascii') != resolved:
+                            resolved = view_argument
+                        # URL parameters from site tree item should be concatenated with those from template.
+                    all_arguments.append(resolved)
+                view_path = view_path[0]
+
             view_arguments = []
-            for view_argument in view_path[1:]:
-                resolved = self.resolve_var(view_argument)
-                # In case of non-ascii data we leave variable unresolved.
-                if isinstance(resolved, unicode):
-                    if resolved.encode('ascii', 'ignore').decode('ascii') != resolved:
-                        resolved = view_argument
-
-                view_arguments.append(resolved)
-
-            # URL parameters from site tree item should be concatenated with
-            # those from template.
-            arguments_couled = tag_arguments + view_arguments
-            view_arguments = []
-
-            for argument in arguments_couled:
+            for argument in all_arguments:
                 argument = str(argument)
-                # To be able to process slug-like strings (strings with "-"'s and "_"'s)
-                # we enclose those in double quotes.
+                # To be able to process slug-like strings (strings with "-"'s and "_"'s) we enclose those in double quotes.
                 if '-' in argument or '_':
                     argument = '"%s"' % argument
                 view_arguments.append(argument)
 
-            view_arguments = ' '.join(view_arguments).strip()
-            view_path = view_path[0]
-            url_pattern = u'%s %s' % (view_path, view_arguments)
+            if DJANGO_VERSION_INT >= 150:  # "new-style" url tag - consider sitetree named urls literals.
+                view_path = "'%s'" % view_path
+
+            url_pattern = u'%s %s' % (view_path, ' '.join(view_arguments))
         else:
-            url_pattern = u'%s' % sitetree_item.url
+            url_pattern = unicode(sitetree_item.url)
 
-        url_pattern = url_pattern.strip()
-
-        # Create 'cache_urls' for this tree.
         tree_alias = sitetree_item.tree.alias
 
         entry_from_cache = self.get_cache_entry('urls', tree_alias)
         if not entry_from_cache:
+            # Create 'cache_urls' for this tree.
             entry_from_cache = {}
             self.set_cache_entry('urls', tree_alias, {})
 
@@ -323,8 +367,7 @@ class SiteTree(object):
             if sitetree_item.urlaspattern:
                 # Form token to pass to Django 'url' tag.
                 url_token = u'url %s as item.url_resolved' % url_pattern
-                url_tag(template.Parser(None),
-                        template.Token(token_type=template.TOKEN_BLOCK, contents=url_token)).render(context)
+                url_tag(template.Parser(None), template.Token(token_type=template.TOKEN_BLOCK, contents=url_token)).render(context)
 
                 # We make an anchor link from an unresolved URL as a reminder.
                 if context['item.url_resolved'] == '':
@@ -374,7 +417,6 @@ class SiteTree(object):
             return ''
         tree_branches = self.resolve_var(tree_branches)
 
-        # Support item addressing both through identifiers and aliases.
         parent_isnull = False
         parent_ids = []
         parent_aliases = []
@@ -382,6 +424,7 @@ class SiteTree(object):
         current_item = self.get_tree_current_item(tree_alias)
         self.tree_climber(tree_alias, current_item)
 
+        # Support item addressing both through identifiers and aliases.
         for branch_id in tree_branches.split(','):
             branch_id = branch_id.strip()
             if branch_id == 'trunk':
@@ -402,17 +445,15 @@ class SiteTree(object):
 
         menu_items = []
         for item in sitetree_items:
-            if item.inmenu and item.hidden == False:
-                if self.check_access(item, context):
-                    if item.parent is None:
-                        if parent_isnull:
-                            menu_items.append(item)
-                    else:
-                        if item.parent.id in parent_ids or item.parent.alias in parent_aliases:
-                            menu_items.append(item)
+            if item.hidden == False and item.inmenu and self.check_access(item, context):
+                if item.parent is None:
+                    if parent_isnull:
+                        menu_items.append(item)
+                else:
+                    if item.parent.id in parent_ids or item.parent.alias in parent_aliases:
+                        menu_items.append(item)
 
         # Parse titles for variables.
-        menu_items = self.parse_titles(menu_items, context)
         menu_items = self.apply_hook(menu_items, 'menu')
         menu_items = self.update_has_children(tree_alias, menu_items, 'menu')
         return menu_items
@@ -430,7 +471,7 @@ class SiteTree(object):
     def check_access(self, item, context):
         """Checks whether user have access to certain item."""
 
-        if item.access_loggedin and not self.global_context['request'].user.is_authenticated():
+        if item.access_loggedin and not self._global_context['request'].user.is_authenticated():
             return False
 
         if item.access_restricted:
@@ -495,7 +536,7 @@ class SiteTree(object):
         return my_template.render(context)
 
     def get_children(self, tree_alias, item):
-        if self.global_context.current_app != 'admin':
+        if self._global_context.current_app != 'admin':
             # We do not need i18n for a tree rendered in Admin dropdown.
             tree_alias = self.resolve_tree_i18n_alias(tree_alias)
         return self.get_cache_entry('parents', tree_alias)[item]
@@ -516,9 +557,9 @@ class SiteTree(object):
         NB: We do not apply any filters to sitetree in admin app.
         """
         items_out = copy(items)
-        if self.global_context.current_app != 'admin':
+        if self._global_context.current_app != 'admin':
             for item in items:
-                no_access = not self.check_access(item, self.global_context)
+                no_access = not self.check_access(item, self._global_context)
                 hidden_for_nav_type = navigation_type is not None and getattr(item, 'in' + navigation_type, False) != True
                 if item.hidden == True or no_access or hidden_for_nav_type:
                     items_out.remove(item)
@@ -546,7 +587,7 @@ class SiteTree(object):
     def breadcrumbs_climber(self, tree_alias, start_from):
         """Climbs up the site tree to build breadcrumb path."""
         if start_from.inbreadcrumbs and start_from.hidden == False and self.check_access(start_from,
-                                                                                         self.global_context):
+                                                                                         self._global_context):
             self.cache_breadcrumbs.append(start_from)
         if hasattr(start_from, 'parent') and start_from.parent is not None:
             self.breadcrumbs_climber(tree_alias, self.get_item_by_id(tree_alias, start_from.parent.id))
@@ -558,7 +599,7 @@ class SiteTree(object):
 
         """
         if context is None:
-            context = self.global_context
+            context = self._global_context
 
         if isinstance(varname, template.FilterExpression):
             varname = varname.resolve(context)
@@ -572,34 +613,6 @@ class SiteTree(object):
 
         return varname
 
-    def parse_titles(self, items, context=None):
-        """Walks through the list of items, and resolves any variable found
-        in a title of an item.
-
-        We deliberately strip down template tokens that are not text or variable.
-        There is definitely no need nor purpose in blocks or comments in a title.
-        Why to think otherwise, if only you're a villain. Joke here.
-
-        Returns the list with resolved titles.
-
-        """
-        if context is None:
-            context = self.global_context
-
-        for item in items:
-            if item.title.find(template.VARIABLE_TAG_START) != -1:
-                my_lexer = template.Lexer(item.title, template.UNKNOWN_SOURCE)
-                my_tokens = my_lexer.tokenize()
-
-                for my_token in my_tokens:
-                    if my_token.token_type not in (template.TOKEN_TEXT, template.TOKEN_VAR):
-                        my_tokens.remove(my_token)
-
-                my_parser = template.Parser(my_tokens)
-                item.title_resolved = my_parser.parse().render(context)
-
-        return items
-
 
 class SiteTreeError(Exception):
     """Exception class for sitetree application."""
