@@ -1,15 +1,14 @@
 from django import template
-from django import VERSION
 from django.templatetags.static import PrefixNode
 
 from ..sitetreeapp import SiteTree
+from ..utils import DJANGO_VERSION_INT
 
 register = template.Library()
 
 # All utility methods are implemented in SiteTree class
 sitetree = SiteTree()
 
-DJANGO_VERSION_INT = int('%s%s%s' % VERSION[:3])
 
 
 @register.tag
@@ -125,16 +124,20 @@ def sitetree_menu(parser, token):
 def sitetree_url(parser, token):
     """This tag is much the same as Django built-in 'url' tag.
     The difference is that after 'for' it should get TreeItem object.
-    And, yes, you can pass some arguments after that TreeItem object.
 
     """
     tokens = token.contents.split()
     tokens_num = len(tokens)
+    as_var = False
 
     if tokens_num >= 3 and tokens[1] == 'for':
+        # TODO Remove additional tag arguments in 1.0.
+        if tokens[-2] == 'as':
+            as_var = tokens[-1]
+            tokens = tokens[:-2]
         sitetree_item = parser.compile_filter(tokens[2])
         tag_arguments = map(parser.compile_filter, tokens[3:])
-        return sitetree_urlNode(sitetree_item, tag_arguments)
+        return sitetree_urlNode(sitetree_item, as_var, tag_arguments)
     else:
         raise template.TemplateSyntaxError, "%r tag should look like {%% sitetree_url for someitem params %%}." % tokens[0]
 
@@ -215,12 +218,16 @@ class sitetree_menuNode(template.Node):
 class sitetree_urlNode(template.Node):
     """Resolves and renders specified url."""
 
-    def __init__(self, sitetree_item, tag_arguments):
+    def __init__(self, sitetree_item, as_var, tag_arguments):
         self.sitetree_item = sitetree_item
         self.tag_arguments = tag_arguments
+        self.as_var = as_var
 
     def render(self, context):
         resolved_url = sitetree.url(self.sitetree_item, self.tag_arguments, context)
+        if self.as_var:
+            context[self.as_var] = resolved_url
+            return ''
         return resolved_url
 
 
